package es.kauron.jstudy.controller;

import es.kauron.jstudy.Main;
import es.kauron.jstudy.model.AppPrefs;
import es.kauron.jstudy.model.TestItem;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.SelectionMode;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.stage.FileChooser;
import javafx.stage.Modality;
import javafx.stage.Stage;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

public class TableController implements Initializable {
    @FXML
    private TableView<TestItem> table;
    @FXML
    private TableColumn<TestItem, String> answerCol, questionCol;
    @FXML
    private Button editButton, duplicateButton, swapButton, testSelectionButton, deleteButton, saveButton;

    private ObservableList<TestItem> data;
    private Controller parent;
    private File file;
    StringProperty name;
    BooleanProperty saved = new SimpleBooleanProperty();

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        table.getSelectionModel().setSelectionMode(SelectionMode.MULTIPLE);
        answerCol.setCellValueFactory(e -> e.getValue().answerProperty());
        questionCol.setCellValueFactory(e -> e.getValue().questionProperty());
        table.getSelectionModel().getSelectedIndices().addListener((ListChangeListener<? super Integer>) obs -> {
            editButton.setDisable(obs.getList().size() != 1);
            swapButton.setDisable(obs.getList().size() < 1);
            deleteButton.setDisable(obs.getList().size() < 1);
            duplicateButton.setDisable(obs.getList().size() < 1);
            testSelectionButton.setDisable(obs.getList().size() < 1);
        });
        saveButton.disableProperty().bind(saved);
    }

    void setData(String name, List<TestItem> list, Controller controller, File file) {
        this.name = new SimpleStringProperty(name);
        this.data = FXCollections.observableArrayList(list);
        this.parent = controller;
        table.setItems(data);
        this.file = file;
        saved.set(file != null);
    }

    List<TestItem> getData() {return new ArrayList<>(data);}
    String getName() {return name.get();}

    @FXML
    protected void onSaveAction(ActionEvent event) {
        if (file == null) {
            FileChooser chooser = new FileChooser();
            chooser.getExtensionFilters().add(new FileChooser.ExtensionFilter("JStudy file", "*.jsdb"));
            if (AppPrefs.lastDir != null) chooser.setInitialDirectory(AppPrefs.lastDir);
            chooser.setInitialFileName(name.get());
            file = chooser.showSaveDialog(table.getScene().getWindow());
            if (!file.getName().endsWith(".jsdb"))
                file = new File(file.getPath() + ".jsdb");
            if (file != null)
                AppPrefs.lastDir = file.getParentFile();
            name.set(file.getName().substring(0, file.getName().lastIndexOf('.')));
        }
        if (file == null) return;
        TestItem.saveTo(file, data);
        saved.set(true);
    }

    @FXML
    protected void onAddAction(ActionEvent event) {
        try {
            FXMLLoader cargador = new FXMLLoader(Main.class.getResource("view/edit.fxml"));
            Parent pRoot = cargador.load();

            ((EditController) cargador.getController()).setList(data, saved);

            Stage stage = new Stage();
            stage.setTitle("New entry");
            stage.setScene(new Scene(pRoot));
            stage.initModality(Modality.APPLICATION_MODAL);
            stage.showAndWait();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @FXML
    protected void onEditAction(ActionEvent event) {
        ObservableList<Integer> list = table.getSelectionModel().getSelectedIndices();
        if (list.size() != 1) return;
        int index = list.get(0);
        try {
            FXMLLoader cargador = new FXMLLoader(Main.class.getResource("view/edit.fxml"));
            Parent root = cargador.load();

            ((EditController) cargador.getController()).setList(table.getItems(), index, saved);

            Stage stage = new Stage();
            stage.setTitle("Editing entry...");
            stage.setScene(new Scene(root));
            stage.initModality(Modality.APPLICATION_MODAL);
            stage.showAndWait();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @FXML
    protected void onSwapAction(ActionEvent event) {
        if (table.getSelectionModel().getSelectedIndices().size() > 0) saved.set(false);
        for (TestItem item : table.getSelectionModel().getSelectedItems()) {
            String question = item.getQuestion();
            item.questionProperty().setValue(item.getAnswer());
            item.answerProperty().setValue(question);
        }
        table.requestFocus();
    }

    @FXML
    protected void onDuplicateAction(ActionEvent event) {
        if (table.getSelectionModel().getSelectedIndices().size() > 0) saved.set(false);
        for (int i : table.getSelectionModel().getSelectedIndices())
            table.getItems().add(new TestItem(table.getItems().get(i)));
        table.requestFocus();
    }

    @FXML
    protected void onDeleteAction(ActionEvent event) {
        if (table.getSelectionModel().getSelectedIndices().size() > 0) saved.set(false);
        for (int i : table.getSelectionModel().getSelectedIndices())
            table.getItems().remove(i);
        table.requestFocus();
    }

    @FXML
    protected void onTestSelectionAction(ActionEvent event) {
        parent.newTest(table.getSelectionModel().getSelectedItems());
    }

    @FXML
    protected void onTestAction(ActionEvent event) {
        parent.newTest(data);
    }
}
